% III.4: Solve advection equation u_t + cu_x = 0 using method of lines
clear all; clc;

% define the spatial domain and temporal domain
a = 0;
b = 2;
t_f = 1;
delta_x = 0.01;

% define Dirichlet boundary condition at x=a
U_a = 0;

x = a : delta_x : b;
n_x = length(x);


% define the initial condition
u0 = 0*x;
u0(find((0.2<=x) & (x<= 0.4))) = 1;

% call ode45
[t, U] = ode45(@(t,U)(advection_MOL_function(t,U,delta_x,U_a)), [0 t_f], u0);


% plot the initial condition
subplot(2,1,1);
plot(x,u0,'linewidth',2);
ylim([0 1.2])
xlabel('x')
ylabel('t')
title('Initial condition')

% plot the exact and numerical solution at t_f
subplot(2,1,2);
plot(x,U(end,:),'r','linewidth',2);
ylim([0 1.2])
xlabel('x')
ylabel('t')
hold on

% the exact solution at time t is u0(x*,t), where
% x* = x - t/(1+x^2)
u_exact = 0*x;
final_ones_indices = find((0.8061<=x) & (x<= 0.9341));
u_exact(final_ones_indices) = ones(size(final_ones_indices));

plot(x,u_exact,'linewidth',2);
legend('MOL','exact')
