% III.4: Solve advection equation u_t + c(x,t)u_x = 0 using Upwind scheme
% where c(x,t) = (1+x^2)/(1+2xt+2x^2+x^4), here 0 < c <= 1
clear all; clc;

delta_x = 0.01;
delta_t = 0.01;

% define the spatial domain and temporal domain
a = 0;
b = 2;
t_f = 1;

% define Dirichlet boundary condition at x=a
U_a = 0;

x = a : delta_x : b;
n_x = length(x);

t = 0 : delta_t : t_f;
n_t = length(t);


% define the initial condition
u0 = 0*x;
u0(find((0.2<=x) & (x<= 0.4))) = 1;

% employ the upwind difference method
U(1,:) = u0;

for n=1:(n_t-1)
    for j=1:n_x
        c_tx = (1+x(j)^2)/(1+2*x(j)*t(n)+2*x(j)^2+x(j)^4);
        if j==1
            U(n+1,j) = U_a;
        else
            U(n+1,j) = U(n,j) - c_tx*delta_t/delta_x*(U(n,j)-U(n,j-1));
        end
    end
end

% plot the initial condition
subplot(2,1,1);
plot(x,u0,'linewidth',2);
ylim([0 1.2])
xlabel('x')
ylabel('t')
title('Initial condition')

% plot the exact and numerical solution at t_f
subplot(2,1,2);
plot(x,U(end,:),'r','linewidth',2);
ylim([0 1.2])
xlabel('x')
ylabel('t')
hold on

% the exact solution at time t is u0(x*,t), where
% x* = x - t/(1+x^2)
u_exact = 0*x;
final_ones_indices = find((0.8061<=x) & (x<= 0.9341));
u_exact(final_ones_indices) = ones(size(final_ones_indices));

plot(x,u_exact,'linewidth',2);
legend('upwind','exact')
