% III.4: Solve heat equation u_t - kappa u_xx = f using the theta method
clear all; clc;

theta = 0.5;
kappa = 1;
delta_x = 0.1;
delta_t = 0.05;

mu = delta_t/delta_x/delta_x


% define the spatial domain and temporal domain
a = 0;
b = 10;
t_f = 2.5;

% define Dirichlet boundary conditions at x=a and x=b
u_a = 0;
u_b = 0;

% define the x and t grid points
x = a : delta_x : b;
n_x = length(x);

t = 0 : delta_t : t_f;
n_t = length(t);

% generate the centered difference differentiation matrix for u_xx
e = 1/delta_x/delta_x*ones(n_x,1);
D2 = spdiags([e,-2*e,e],[-1:1],n_x,n_x);

% build the system matrix, A
A = speye(n_x) - theta*delta_t*kappa*D2;

% set first and last rows of A
A(1,:)     = zeros(1,n_x);
A(1,1)     = 1;
A(n_x,:)   = zeros(1,n_x);
A(n_x,n_x) = 1;

% define the initial condition
u0 = 0*x;
nonzero_start = round(n_x*0.4);
nonzero_end   = round(n_x*0.6);
u0(nonzero_start:nonzero_end) = ones(1,nonzero_end-nonzero_start+1);

% solve for U using the theta method
U(1,:) = u0;

for n=1:(n_t-1)
    % define the right-hand side vector
    rhs = (speye(n_x) + kappa*(1-theta)*delta_t*D2)*U(n,:)';
    
    % set the first and last entries of rhs to
    % impose boundary conditions
    rhs(1)   = u_a;
    rhs(n_x) = u_b;
    
    % solve for U
    U(n+1,:) = A\rhs;
end

% plot the solution
figure(1)
[xx,tt] = meshgrid(x,t);
waterfall(xx,tt,U)
xlabel('x')
ylabel('t')
