% III.3: ODE Boundary Value Problem using finite difference method
% solve -alpha u'' + beta u' + gamma u = f, u(a) = c_1, u(b) = c_2
clear all; clc;

% define parameters
alpha = 0.5;
beta = -2;
gamma = 0.1;

c_1 = 0;
c_2 = 0;

a = 0; b = 1;

% we consider n grid points on the interval [a,b]
n = 100;
h = (b-a)/(n-1);
x = linspace(a,b,n)';

% generate the centered difference differentiation matrix for u'
e = 1/2/h*ones(n,1);
D1 = spdiags([-e,e],[-1,1],n,n);

% generate the centered difference differentiation matrix for u''
e = 1/h/h*ones(n,1);
D2 = spdiags([e,-2*e,e],[-1:1],n,n);

% build the system matrix, A
A = -alpha*D2 + beta*D1 + gamma*eye(n);

% define the right-hand side vector
F = -alpha * exp(x) .* (4*pi*cos(2*pi*x) + (1-4*pi^2)*sin(2*pi*x)) + ...
     beta*exp(x).* (sin(2*pi*x) + 2*pi*cos(2*pi*x)) + ...
     gamma*exp(x).*sin(2*pi*x);

% --- Impose Dirichlet boundary conditions (start) ---
% set first and last rows of A
A(1,:)     = zeros(1,n);
A(1,1)     = 1;
A(n,:)     = zeros(1,n);
A(n,n)     = 1;

% set first and last rows of F
F(1) = c_1;
F(n) = c_2;
% --- Impose Dirichlet boundary conditions (end) ---

% solve the linear system
U = A\F;

% get the exact solution
u_exact = exp(x).*sin(2*pi*x);

% plot the exact and approximate solutions
plot(x,u_exact,'k','linewidth',2)
hold on
plot(x,U,'r--','linewidth',2)
legend('exact u','U')
