% III.4: Solve Poisson equation \Delta u = f with zero Dirichlet BCs
clear all; clc;

tic
% define the right-hand side function
f = @(x,y)(-exp(-(x-0.25).^2 - (y-0.5).^2));

% define the domain and mesh size
a = 0; b = 1;
n_x = 101;
n_y = n_x;
h = (b-a)/(n_x-1);

% generate the centered difference differentiation matrix for the Laplacian
e = 1/h/h*ones(n_x*n_y,1);
D2 = spdiags([e,e,-4*e,e,e],[-n_x,-1:1,n_x],n_x*n_y,n_x*n_y);

% generate the right-hand side vector
rhs = zeros(n_x*n_y,1);
size(rhs)
for i=1:n_x
    for j=1:n_y
        x_i = (i-1)*h;
        y_j = (j-1)*h;
        rhs((j-1)*n_x+i) = f(x_i,y_j);
    end
end

% now apply boundary conditions
% y = a indices
y_min_boundary_indices = 1:n_x;
% y = b indices
y_max_boundary_indices = (n_y-1)*n_x+1:n_y*n_x;
% x = a indices
x_min_boundary_indices = n_x+1:n_x:(n_y-2)*n_x+1;
% x = b indices
x_max_boundary_indices = 2*n_x:n_x:(n_y-1)*n_x;

dirichlet_indices = [y_min_boundary_indices,y_max_boundary_indices, ...
                     x_min_boundary_indices,x_max_boundary_indices];
dirichlet_indices = sort(unique(dirichlet_indices));

% overwrite D2 and rhs as appropriate
for index = dirichlet_indices
    D2(index,:) = zeros(1,n_x*n_y);
    D2(index,index) = 1;
    
    rhs(index) = 0;
end

% solve for U
U = D2\rhs;

toc

% get the meshgrid, for plotting
xx = zeros(n_x,n_y);
yy = zeros(n_x,n_y);

% reshape U so that we can plot it wrt xx,yy
UU = 0*xx;
for i=1:n_x
    for j=1:n_y
        xx(i,j) = (i-1)*h;
        yy(i,j) = (j-1)*h;
        UU(i,j) = U((j-1)*n_x+i);
    end
end

% plot the solution
surf(xx,yy,UU);
colorbar
xlabel('x')
ylabel('y')

